import mitt from 'mitt';
import * as React from 'react';
import { StyleSheet, View, findNodeHandle, } from 'react-native';
import { TourGuideContext } from '../components/TourGuideContext';
import { useIsMounted } from '../hooks/useIsMounted';
import * as utils from '../utilities';
import { Modal } from './Modal';
import { OFFSET_WIDTH } from './style';
const { useMemo, useEffect, useState, useRef, useCallback } = React;
const MAX_START_TRIES = 120;
export const TourGuideProvider = ({ children, wrapperStyle, labels, tooltipComponent, tooltipStyle, androidStatusBarVisible, backdropColor, animationDuration, maskOffset, borderRadius, verticalOffset, startAtMount = false, }) => {
    const [visible, setVisible] = useState(undefined);
    const [currentStep, updateCurrentStep] = useState();
    const [steps, setSteps] = useState({});
    const [canStart, setCanStart] = useState(false);
    const [scrollView, setScrollView] = useState(undefined);
    const startTries = useRef(0);
    const mounted = useIsMounted();
    const eventEmitter = useMemo(() => new mitt(), []);
    const modal = useRef();
    useEffect(() => {
        if (mounted && visible === false) {
            eventEmitter.emit('stop');
        }
    }, [visible]);
    useEffect(() => {
        if (visible || currentStep) {
            moveToCurrentStep();
        }
    }, [visible, currentStep]);
    useEffect(() => {
        if (mounted) {
            if (Object.entries(steps).length > 0) {
                setCanStart(true);
                if (startAtMount) {
                    start();
                }
            }
            else {
                setCanStart(false);
            }
        }
    }, [mounted, steps]);
    const moveToCurrentStep = async () => {
        var _a;
        const size = await currentStep.target.measure();
        await ((_a = modal.current) === null || _a === void 0 ? void 0 : _a.animateMove({
            width: size.width + OFFSET_WIDTH,
            height: size.height + OFFSET_WIDTH,
            left: size.x - OFFSET_WIDTH / 2,
            top: size.y - OFFSET_WIDTH / 2 + (verticalOffset || 0),
        }));
    };
    const setCurrentStep = async (step) => {
        var _a, _b;
        updateCurrentStep(() => {
            eventEmitter.emit('stepChange', step);
            return step;
        });
        if (scrollView) {
            const adjustment = (_a = step === null || step === void 0 ? void 0 : step.scrollAdjustment) !== null && _a !== void 0 ? _a : 0;
            if (step === null || step === void 0 ? void 0 : step.insideScroll) {
                await step.wrapper.measureLayout(findNodeHandle(scrollView === null || scrollView === void 0 ? void 0 : scrollView.current), (_x, y, _w, h) => {
                    var _a;
                    const yOffset = y > 0 ? y + adjustment - h / 2 : adjustment;
                    (_a = scrollView === null || scrollView === void 0 ? void 0 : scrollView.current) === null || _a === void 0 ? void 0 : _a.scrollTo({ y: yOffset, animated: false });
                });
            }
            else {
                (_b = scrollView.current) === null || _b === void 0 ? void 0 : _b.scrollTo({ y: adjustment, animated: false });
            }
        }
    };
    const getNextStep = (step = currentStep) => utils.getNextStep(steps, step);
    const getPrevStep = (step = currentStep) => utils.getPrevStep(steps, step);
    const getFirstStep = (currentStep) => utils.getFirstStep(steps, currentStep === null || currentStep === void 0 ? void 0 : currentStep.tag);
    const getLastStep = (currentStep) => utils.getLastStep(steps, currentStep === null || currentStep === void 0 ? void 0 : currentStep.tag);
    const isFirstStep = useMemo(() => currentStep === getFirstStep(currentStep), [currentStep]);
    const isLastStep = useMemo(() => currentStep === getLastStep(currentStep), [currentStep]);
    const next = () => setCurrentStep(getNextStep());
    const prev = () => setCurrentStep(getPrevStep());
    const stop = () => {
        var _a;
        setVisible(false);
        setCurrentStep(undefined);
        if (scrollView) {
            (_a = scrollView.current) === null || _a === void 0 ? void 0 : _a.scrollTo({ y: -300, animated: false });
            setScrollView(undefined);
        }
    };
    const registerStep = (step) => {
        setSteps((previousSteps) => {
            return {
                ...previousSteps,
                [step.name]: step,
            };
        });
    };
    const unregisterStep = (stepName) => {
        if (!mounted) {
            return;
        }
        setSteps((previousSteps) => {
            return Object.entries(previousSteps)
                .filter(([key]) => key !== stepName)
                .reduce((obj, [key, val]) => Object.assign(obj, { [key]: val }), {});
        });
    };
    const getCurrentStep = () => currentStep;
    const start = useCallback(async (flowTag, scrollView) => {
        const currentStep = flowTag
            ? Object.values(steps)
                .filter((step) => flowTag === step.tag)
                .sort((a, b) => a.order - b.order)[0]
            : getFirstStep();
        if (startTries.current > MAX_START_TRIES) {
            startTries.current = 0;
            return;
        }
        if (scrollView) {
            setScrollView(scrollView);
        }
        if (!currentStep) {
            startTries.current += 1;
            requestAnimationFrame(() => start(flowTag));
        }
        else {
            eventEmitter.emit('start');
            await setCurrentStep(currentStep);
            setVisible(true);
            startTries.current = 0;
        }
    }, [steps, scrollView]);
    const valueProvider = useMemo(() => {
        return {
            eventEmitter,
            registerStep,
            unregisterStep,
            getCurrentStep,
            start,
            stop,
            canStart,
            setScrollView,
        };
    }, [start, steps]);
    return (React.createElement(View, { style: [styles.container, wrapperStyle] },
        React.createElement(TourGuideContext.Provider, { value: valueProvider },
            children,
            React.createElement(Modal, Object.assign({ ref: modal }, {
                next,
                prev,
                stop,
                visible,
                isFirstStep,
                isLastStep,
                currentStep,
                labels,
                tooltipComponent,
                tooltipStyle,
                androidStatusBarVisible,
                backdropColor,
                animationDuration,
                maskOffset,
                borderRadius,
            })))));
};
const styles = StyleSheet.create({
    container: {
        flex: 1,
    },
});
